from rest_framework import permissions


class KongOAuthPermission(permissions.BasePermission):
    """
    Determines if user has the correct permissions for a class based view,

    add to class based view (Not that Function Based Views can use the
    permission_required decorator with KongOAuthPermission):

    permission_classes = (KongOAuthPermission,)
    permission_list = ['group_1', 'group_2]

    this will then check a user and see if they have the required permissions
    """

    def has_permission(self, request, view):
        if view.permission_list:
            for group in view.permission_list:
                if request.user.has_perm(group):
                    return True
        return False

    def has_object_permission(self, request, view, obj):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        object_type = obj.__class__.__name__.lower()
        if request.method is 'GET':
            for permission in view.permissions_map['list']:
                if permission in request.user.get_all_permissions():
                    return True
        if request.method is 'POST':
            for permission in view.permissions_map['create']:
                if permission in request.user.get_all_permissions():
                    return True
        return False



from django.contrib.auth.models import User, Permission, ContentType
from rest_framework.test import APIRequestFactory
from ..permissions import KongOAuthPermission
from unittest import TestCase
from mock import MagicMock
from faker import Faker


def mock_has_perm(permission):
    if permission == 'all':
        return True
    return False


class KongOAuthPermissionTestCase(TestCase):

    def setUp(self):
        f = Faker()
        self.rf = APIRequestFactory()
        self.permission = KongOAuthPermission()
        self.request = MagicMock(user=MagicMock())
        self.view = MagicMock()
        self.request.user = User.objects.create_user(username=f.first_name())

    def test_has_group_permission(self):
        self.view.permission_list = ['.all']
        self.content_type = ContentType.objects.create(
            app_label='', model='unused')
        permission = Permission.objects.create(codename='all',
                                               name='all',
                                               content_type=self.content_type)
        self.request.user.user_permissions.add(permission)
        self.assertTrue(self.permission.has_permission(self.request,
                                                       self.view),
                        msg='Permission Failed')
        ContentType.delete(self.content_type)

    def test_has_obj_permission(self):
        self.view.permissions_map = {'list': ['.add_user', 'other_group']}
        self.content_type = ContentType.objects.create(
            app_label='', model='unused')
        self.request.method = 'GET'
        permission = Permission.objects.create(codename='add_user',
                                               name='add_user',
                                               content_type=self.content_type)
        obj = User.objects.create_user(username='test_object_user')
        self.request.user.user_permissions.add(permission)
        self.assertTrue(self.permission.has_object_permission(self.request,
                                                              self.view,
                                                              obj),
                        msg='Permission Failed')
        ContentType.delete(self.content_type)
